@@ -23,14 +23,14 @@
 typedef unsigned int MD4_u32plus;
 
 typedef struct {
+	MD4_u32plus A, B, C, D;
 	MD4_u32plus lo, hi;
-	MD4_u32plus a, b, c, d;
 	unsigned char buffer[64];
 	MD4_u32plus block[16];
 } MD4_CTX;
 
 extern void MD4_Init(MD4_CTX *ctx);
-extern void MD4_Update(MD4_CTX *ctx, void *data, unsigned long size);
+extern void MD4_Update(MD4_CTX *ctx, const void *data, unsigned long size);
 extern void MD4_Final(unsigned char *result, MD4_CTX *ctx);
 
 #endif
